*/////////////////////////////////////////////////////////////////////////////
*//                                                                         //
*// Actual version rewritten by S. Jadach, Nov 1997.                        //
*//                                                                         //
*// Universal random number generator proposed by MARSAGLIA and ZAMAN       //
*// in report FSU-SCRI-87-50                                                //
*//        modified by F. James, 1988 and 1989, to generate a vector        //
*//        of pseudorandom numbers rvec of length lenv, and to put in       //
*//        the COMMON block everything needed to specify currrent state,    //
*//        and to add input and output entry points rmarin, rmarut.         //
*// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   //
*// ++  CALLing sequences for ranmar:                                  ++   //
*// ++      CALL ranmar (rvec, len)   returns a vector rvec of len     ++   //
*// ++                   32-bit random floating point numbers between  ++   //
*// ++                   zero and one.                                 ++   //
*// ++      CALL rmarin(i1,n1,n2)   initializes the generator from one ++   //
*// ++                  32-bit integer i1, and number counts n1,n2     ++   //
*// ++                  (for initializing, set n1=n2=0, but to restart ++   //
*// ++                    a previously generated sequence, use values  ++   //
*// ++                    output by rmarut)                            ++   //
*// ++      CALL rmarut(i1,n1,n2)   outputs the value of the original  ++   //
*// ++                  seed and the two number counts, to be used     ++   //
*// ++                  for restarting by initializing to i1 and       ++   //
*// ++                  skipping n2*100000000+n1 numbers.              ++   //
*// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   //
*//                                                                         //
*//         Initializing routine for ranmar, may be called before           //
*//         generating pseudorandom numbers with ranmar. the input          //
*//         values should be in the ranges:  0<=ijklin<=900 000 000         //
*//                                          0<=ntotin<=999 999 999         //
*//                                          0<=ntot2n<<999 999 999!        //
*// to get the standard values in MARSAGLIA's paper, ijklin=54217137        //
*//                                            ntotin,ntot2n=0              //
*//                                                                         //
*/////////////////////////////////////////////////////////////////////////////


      SUBROUTINE PseuMar_MakeVec(rvec,lenv)
*/////////////////////////////////////////////////////////////////////////////
*//                                                                         //
*/////////////////////////////////////////////////////////////////////////////
      IMPLICIT NONE
      INCLUDE 'PseuMar.h'
*
      REAL              rvec(*)
      INTEGER lenv
      INTEGER ijkl_new, ntot_new,ntot2_new
      REAL              zuni,uni
      INTEGER ivec
*-------------------------------------------------------------------------
      IF (m_iwarm  .NE.  9000009)  THEN
* Default initialization. User has called ranmar without rmarin.
         ijkl_new  = 54217137
         ntot_new  = 0
         ntot2_new = 0
         CALL PseuMar_Initialize(ijkl_new, ntot_new,ntot2_new)
      ENDIF

* Normal entry to generate lenv random numbers
      DO ivec= 1, lenv
         uni = m_U(m_i97)-m_U(m_j97)
         IF (uni  .LT.  0.)  uni=uni+1.
         m_U(m_i97) = uni
         m_i97 = m_i97-1
         IF (m_i97  .EQ.  0)  m_i97=97
         m_j97 = m_j97-1
         IF (m_j97  .EQ.  0)  m_j97=97
         m_C = m_C - m_CD
         IF (m_C  .LT.  0.)   m_C=m_C+m_CM
         uni = uni-m_C
         IF (uni  .LT.  0.) uni=uni+1.
         rvec(ivec) = uni
* Replace exact zeros by uniform distr. *2**-24
         IF (uni  .EQ.  0.)  THEN
            zuni = m_twom24*m_U(2)
*     an exact zero here is very unlikely, but let's be safe.
            IF (zuni  .EQ.  0.) zuni= m_twom24*m_twom24
            rvec(ivec) = zuni
         ENDIF
      ENDDO
      m_ntot  = m_ntot + lenv
      IF (m_ntot  .GE.  modcns)  THEN
         m_ntot2  =  m_ntot2 + 1
         m_ntot   =  m_ntot - modcns
      ENDIF
      END


